/*

This file replaces ALL the Hi-Tech .h files which are supplied with the 
standard v7 H8/300 C compiler.  

The reason for this approach is that the supplied Hi-Tech .h files
are largely common across their different cross-compilers and thus contain
a great deal of material which is irrelevant to the KD485 and its inclusion 
could cause confusion.

This file also contains the prototypes for all those functions that are 
located in the H8 on-chip PROM; these include KD485-specific C extensions.  

You should not edit this file, for maintenance reasons.  Rather, place any 
additions into a separate .h file.

MOD RECORD

13/8/94		Created for PPC.  (tab stop for editing = 2).
4/12/94		Moved over to KD485.
4/5/95		get_vecbase added.
19/6/95		waggle_pin added.
24/6/95		toupper, tolower have 'char' parms and returns.
16/6/98		dprintf() and readhex() and rtc functions added.


*/


/* From Hi-Tech ctype.h */

#define	_U	0x01
#define	_L	0x02
#define	_N	0x04
#define	_S	0x08
#define _P	0x10
#define _C	0x20
#define	_X	0x40
#define	_B	0x80

	extern const unsigned char	_ctype_[];

// Note: these macros can be used in *user programs only, because the table
//       _ctype_ is in the kd485.lib library.

#define	isupper(c)	((_ctype_+1)[(unsigned char)(c)]&_U)
#define	islower(c)	((_ctype_+1)[(unsigned char)(c)]&_L)
#define	isxdigit(c)	((_ctype_+1)[(unsigned char)(c)]&(_N|_X))
#define ispunct(c)	((_ctype_+1)[(unsigned char)(c)]&_P)
#define isalnum(c)	((_ctype_+1)[(unsigned char)(c)]&(_U|_L|_N))
#define isprint(c)	((_ctype_+1)[(unsigned char)(c)]&(_P|_U|_L|_N|_B))
#define isgraph(c)	((_ctype_+1)[(unsigned char)(c)]&(_P|_U|_L|_N))
#define iscntrl(c)	((_ctype_+1)[(unsigned char)(c)]&_C)
#define isascii(c)	(!((c)&~0x7F))
#define toascii(c)	((c)&0x7F)



/* From Hi-Tech float.h */

/*	Characteristics of floating types */

#define	DBL_RADIX				2		/* radix of exponent for a double */
#define	DBL_ROUNDS			1		/* doubles round when converted to int */
#define	FLT_RADIX				2		/* radix of float exponent */
#define	FLT_ROUNDS			1		/* float also rounds to int */

#define	FLT_MANT_DIG		24							/* 24 bits in mantissa */
#define	DBL_MANT_DIG		24							/* ditto for double */
#define	FLT_EPSILON			1.19209290e-07	/* smallest x, x+1.0 != 1.0 */
#define	DBL_EPSILON			1.19209290e-07	/* smallest x, x+1.0 != 1.0 */
#define	FLT_DIG					6								/* decimal significant digs */
#define	DBL_DIG					6
#define	FLT_MIN_EXP			-125
#define	DBL_MIN_EXP			-125
#define	FLT_MIN					1.175494e-38 		/* smallest floating number */
#define	DBL_MIN					1.175494e-38
#define	FLT_MIN_10_EXP	-37
#define	DBL_MIN_10_EXP	-37
#define	FLT_MAX_EXP			128							/* max binary exponent */
#define	DBL_MAX_EXP			128
#define	FLT_MAX					6.805647e+38		/* max floating number */
#define	DBL_MAX					6.805647e+38
#define	FLT_MAX_10_EXP	38							/* max decimal exponent */
#define	DBL_MAX_10_EXP	38


/*	long double equates to double */


#define	LDBL_MANT_DIG		DBL_MANT_DIG
#define	LDBL_EPSILON		DBL_EPSILON
#define	LDBL_DIG				DBL_DIG
#define	LDBL_MIN_EXP		DBL_MIN_EXP
#define	LDBL_MIN				DBL_MIN
#define	LDBL_MIN_10_EXP	DBL_MIN_10_EXP
#define	LDBL_MAX_EXP		DBL_MAX_EXP
#define	LDBL_MAX				DBL_MAX
#define	LDBL_MAX_10_EXP	DBL_MAX_10_EXP


/* From Hi-Tech hitech.h */


#define	ANSI	1

/*	shorthand types */

#define	uchar		unsigned char
#define	ulong		unsigned long
#define	ushort	unsigned short

/* useful, tuneable types. Change only if:

	1) compiler does not support type, e.g. unsigned char.
	2) compiler generates bad code for a particular type.
	3) a larger type would generate faster code, e.g. byte counters
	   on the 65816 are inefficient code-wise.
 */

#define	BOOL	unsigned char		/* boolean variable. Any integral type
					   will do. */
#define	FAST	char			/* fast, small counter. Must permit
					   values -128 to 127 but may be larger. */
#define	UFAST	unsigned char		/* fast, small unsigned counter. Must
					   permit values 0-255 at least */
#define	BYTE	unsigned char		/* sizeof(BYTE) must == 1 */
#define	INT_16	short			/* signed, >= 16 bits */
#define	UINT_16	unsigned short		/* unsigned, >= 16 bits */
#define	INT_32	long			/* signed, >= 32 bits */
#define	UINT_32	unsigned long		/* unsigned, >= 32 bits */


/* From Hi-Tech limits.h */

/*	Characteristics of integral types */

#define	CHAR_BIT	8										/* bits per char */
#define	CHAR_MAX	127									/* max value of a char */
#define	CHAR_MIN	-128								/* min value */
#define	SCHAR_MAX	CHAR_MAX						/* chars are signed */
#define	SCHAR_MIN	CHAR_MIN
#define	UCHAR_MAX	255									/* for unsigned chars */
#define	SHRT_MAX	32767								/* max value of a short */
#define	SHRT_MIN	(int)-32768
#define	USHRT_MAX	65535								/* unsigned short */

#define	INT_MAX		32767								/* max for int */
#define	INT_MIN		(int)-32768					/* min for int */
#define	UINT_MAX	65535								/* unsigned int */
#define	LONG_MAX	2147483647					/* max value of long */
#define	LONG_MIN	(long)-2147483648		/* min value */
#define	ULONG_MAX	4294967295					/* unsigned long */


/* From Hi-Tech math.h */

 double fabs(double), floor(double), ceil(double), modf(double, double *);
 double sqrt(double), atof(const char *);
 double sin(double), cos(double), tan(double);
 double asin(double), acos(double), atan(double), atan2(double, double);
 double frexp(double, int *), ldexp(double, int);
 double log(double), log10(double), pow(double, double), exp(double);
 double sinh(double), cosh(double), tanh(double);
 double	eval_poly(double, const double *, int);


/* From Hi-Tech setjmp.h */

typedef	int	jmp_buf[2];

	int	setjmp(jmp_buf);
 void	longjmp(jmp_buf, int);


/* From Hi-Tech stdarg.h */

/*	Macros for accessing variable arguments */

#ifndef	_STDARG
typedef void *	va_list[1];
#define	va_start(ap, parmn)	*ap = (char *)&parmn + sizeof parmn
#define	va_arg(ap, type)	(*(*(type **)ap)++)
#define	_STDARG
#endif
#define	va_end(ap)


/* From Hi-Tech stddef.h */

#ifndef	_STDDEF
typedef	int		ptrdiff_t;	/* result type of pointer difference */
typedef	unsigned	size_t;		/* type yielded by sizeof */
typedef unsigned short	wchar_t;	/* wide char type */
#define	_STDDEF
#define	offsetof(ty, mem)	((int)&(((ty *)0)->mem))
#endif	_STDDEF

#ifndef	NULL
#define	NULL	((void *)0)
#endif	NULL

#define	EOF		(-1)


/* From Hi-Tech stdlib.h */

#define	RAND_MAX	32767		/* max value returned by rand() */

double	atof(const char *);
int			atoi(const char *);
unsigned	xtoi(const char *);
long		atol(const char *);
int			rand(void);
void		srand(unsigned int);
void *	calloc(size_t, size_t);

void *	malloc(size_t);
void		free(void *);
void *	realloc(void *, size_t);

void		qsort(void *, size_t, size_t, int (*)(const void *, const void *));
void *	bsearch(const void *, void *, size_t, size_t, int(*)(const void *, const void *));
int			abs(int);
long		labs(long);


/* From Hi-Tech string.h */

/*	String functions */

void *	memcpy(void *, const void *, size_t);
void *	memmove(void *, const void *, size_t);
char *	strcpy(char *, const char *);
char *	strncpy(char *, const char *, size_t);
char *	strcat(char *, const char *);
char *	strncat(char *, const char *, size_t);
int			memcmp(const void *, const void *, size_t);
int			strcmp(const char *, const char *);
int			stricmp(const char *, const char *);
int			strncmp(const char *, const char *, size_t);
size_t	strcoll(char *, size_t, char *);
void *	memchr(const void *, int, size_t);
size_t	strcspn(const char *, const char *);
char *	strpbrk(const char *, const char *);
size_t	strspn(const char *, const char *);
char *	strstr(const char *, const char *);
char *	strtok(char *, const char *);
char *	strdup(const char *);
void *	memset(void *, int, size_t);
char *	strerror(int);
size_t	strlen(const char *);
char *	strchr(const char *, int);
char *	strrchr(const char *, int);


/* From Hi-Tech sys.h */

void *	sbrk(size_t);
void *	brk(void *);


/* From Hi-Tech intrpt.h */

#define	ei()	asm("	andc #H'7F,ccr")
#define	di()	asm("	orc	#H'80,ccr")


// From Hi-Tech time.h
// This applies only if the optional RTC is fitted. This is an optional
// KD485-PROG feature which needs rtc.obj, rtc2.obj to be linked in.

// Structure used by setrtc and getrtc.  Based on K & R.
// Members marked ** are ignored by KD485 RTC.

struct tm
{
	int	tm_sec;			  //	seconds after the minute  (0..59)
	int	tm_min;			  //	minutes after the hour    (0..59)
	int	tm_hour;		  //	hours since midnight      (0..23)
	int	tm_mday;		  //	day of the month          (1..31)
	int	tm_mon;			  //	months since January      (0..11)
	int	tm_year;		  //	years since 1900          (50..149)
	int	tm_wday;		  //	days since Sunday         (0..6)
	int	tm_yday;		  //	days since Jan 1          (0..365)  **
	int	tm_isdst;		  //	Daylight Saving Time flag (0..1)    **
};

 int	time_zone (void);	/* minutes WESTWARD of Greenwich, =0 */
 long	time(long *);	/* seconds since 00:00:00 Jan 1 1970 */
 int	stime(long *);	/* set time */
 char *	asctime(struct tm *);	/* converts struct tm to ascii time */
 char *	ctime(long *);	/* current local time in ascii form */
 struct tm *	gmtime(long *);	/* Universal time */
 struct tm *	localtime(long *);	/* local time */



/***** KD485-SPECIFIC STUFF *****/

#define port1 	1
#define port2 	2

#define	stdin		port1
#define	stdout	port1
#define	stderr	port1

#define	TOTAL_PORTS 2

#define	PAR_EE_START		0x4000	// start of user program area
#define	PAR_EE_SIZE			0x8000	// size  of user program area
#define USER_EE_START		0xB830	// base of 2000-byte "user" area (nvwrblk)


/***** SUPPORTED C FUNCTIONS *****/

int						main				( void );
int						fprintf			( int pt, char * fmt, ... );
int 					vfprintf 		( int pt, char * fmt, va_list ap );
int						sprintf			( char * bufname, char * fmt, ... );
int 					vsprintf		( char * bufname, char * fmt, va_list ap );
int						sscanf			( char * bufname, char * fmt, ... );
int						vsscanf			( char * bufname, char * fmt, va_list ap );


/* these int+long-only versions are in the PROM-based runtime library */

int						ilfprintf		( int pt, char * fmt, ... );
int 					ilvfprintf 	( int pt, char * fmt, va_list ap );
int						ilsprintf		( char * bufname, char * fmt, ... );
int 					ilvsprintf	( char * bufname, char * fmt, va_list ap );


char					toupper			( char );
char					tolower			( char );
int						isalpha			( char );
int						isdigit			( char );
int						isspace			( char );


/***** KD485-SPECIFIC FUNCTIONS *****/

int 					fwritec 		( int pt,  char * bufname, int charstowrite );
int 					freadc 			( int pt,  char * bufname, int charstoread );
char 					fgetc				( int pt );
int 					fgetcnd			( int pt );
int						fputc				( int pt, char chartowrite );
unsigned char	sumbuf			( char * bufname, int buflen );
unsigned char	xorbuf			( char * bufname, int buflen );
unsigned int	crcbuf			( char * bufname, int buflen, int initvalue );
int						testbit16		( int bitnum,  unsigned int  value );
int						testbit8		( int bitnum,  unsigned char value );
unsigned int	rotleft16		( int rotates, unsigned int  value );
unsigned int	rotright16	( int rotates, unsigned int  value );
unsigned char	rotleft8		( int rotates, unsigned char value );
unsigned char	rotright8		( int rotates, unsigned char value );
unsigned char	testset			( char * address );
void					testsetw		( char * address );
int						ipqcount		( int pt );
int						ipqclear		( int pt );
int						opqspace		( int pt );
int						opqcount		( int pt );
int						loadtimer		( int timernum, unsigned int timervalue );
unsigned int	readtimer		( int timernum );
void					getsw				( struct sw_t * anyname );
void					uled				( int status );
int						sysdata			( char * bufname );
int						brkout			( int pt, unsigned int duration );
int						setportcfg	( struct sp_t * anyname );	// uses timer 7!
int						getportcfg	( struct sp_t * anyname );
int						nvwrblk			( char * src, unsigned int dest, unsigned int blksize );
int						nvwrblku		( char * src, unsigned int dest, unsigned int blksize );
int						chk_par_cs	( int mode );
void					waggle_pin	( void );
void					wdg_disable	( void );
void 					wdg_enable	( void );
void					wdg_mode		( int mode );
void					wdg_pulse		( void );
void					drvctrl			( int pt, int enable_state );
void					wait_485		( int pt, int drvstate );		// uses timer 7 @<9600 baud!
unsigned char * geterrs		( void );
char *				getvecbase 	( void );
void *				setvec			( int vecnum, void * isr );
int						getrts			( void );
void					exit				( void );
void 					dfwritec		( char * buffer, int length );
int						readhex			( void );
int 					getrtc  		( struct tm * anyname );
int 					setrtc  		( struct tm * anyname );
unsigned char	rtc_read_byte  				( int rtc_address );
void					rtc_write_byte 				( int rtc_address, unsigned char data );
void					rtc_read_block_ram 		( char * buffer );
void					rtc_write_block_ram		( char * buffer );
void					rtc_read_block_clock 	( char * buffer );
void					rtc_write_block_clock	( char * buffer );



/* structure used by getsw */

struct sw_t
{
	int	sw_sw1;
	int	sw_sw2;
	int	sw_sw3;
};



/* structure used by setportcfg & getportcfg */

struct sp_t
{
	int sp_port;			/* 1,2 = p1,2       	 						*/
	int sp_brate;			/* 0,1.....20 (see table below)   */
	int sp_bword;			/* 2,3 = 7,8 b/word								*/
	int sp_pty;				/* 0,1,2	= none,even,odd parity	*/
	int sp_sbits;			/* 0,1	= 1,2 stop bits						*/
	int sp_rxrtshs;		/* reserved												*/
	int sp_rxdtrhs;		/* reserved												*/
	int sp_rxxhs;			/* 0,1	= off,automatic						*/
	int sp_txctshs;		/* reserved												*/
	int sp_txdsrhs;		/* reserved												*/
	int sp_txxhs;			/* 0,1	= off,automatic						*/
};

/*

sp_brate baud rate					 sp_brate baud rate

   0		    30	  						  11		  2000	 
   1		    37.5  						  12		  2400
   2		    50	  						  13		  3600	 
   3		    75	  						  14		  4800
   4		   100	  						  15		  7200	 
   5		   110	  						  16		  9600
   6		   134.5  						  17		 19200
   7		   150	  						  18		 38400
   8		   300	  						  19		 57600	 
   9		   600								  20		115200	
  10		  1200


*/

// Enable comp-time checking of printf etc templates (requires -w-1 or lower)

#pragma	printf_check(sprintf)
#pragma	printf_check(vsprintf)
#pragma	printf_check(fprintf)
#pragma	printf_check(vfprintf)
#pragma	printf_check(ilsprintf)
#pragma	printf_check(ilvsprintf)
#pragma	printf_check(ilfprintf)
#pragma	printf_check(ilvfprintf)


