program kdtest0;
{$L-,O-}
{

Test program for KD485 Mode 0.

P4 connects to KD485 RS485 port (P2).
P2 connects to KD485 RS232 port (P1).

Requires RTS to be DISABLED as automatic handshake on P4 (drives K485-99 RTS).
Requires RTS to be DISABLED as automatic handshake on P2 (drives KD485 RTSIN).

P4 has a K485-99 in it.

The value "rtsdel" would normally be just 2-3ms, but if e.g. rs232 rate is 38k 
and rs485 rate is 115k then it needs to be a lot longer, because the KD485 M0
program just copies RTSIN -> driver state, without taking account of data.


MOD RECORD

7/7/95	Created.



}

const 
	rs232 	= port2;
	rs485 	= port4;	{ use P3/P4, for optimum opqcount operation }
	term  	= port1;
	bufmax	= 30;
	rtsdel  = 10;		{ needs tweaking if rs232 & rs485 baud rates differ }
	
var i,errcnt1,errcnt2,errcnt3,errcnt4,errcnt5,errcnt6,bufupd:integer;
	ch:char;
	hskcontrol:sethsktype;
	pass:real;
	buffer:array[0..bufmax] of char;
	

procedure setrts(p:text; state:boolean);
{ set RTS on port p to specified state }
var	ret:integer;
	hskcontrol:sethsktype;
begin
	with hskcontrol do begin
		hppc_port:=p;
		rts_out:=state; 		{ set RTS to specified state }
		dtr_out:=true; 			{ required for K485 power }
		cdo_out:=false;			{ not used }
	end;
	ret:=sethsk(hskcontrol);	{ execute the new states }
end;


begin

setrts(rs485,false);			{ also sets DTR high }
setrts(rs232,false);

pass:=1;
errcnt1:=0;
errcnt2:=0;
bufupd:=0;

repeat


	{ fill buffer with random data, change it every so often }

	if bufupd=0 then 
	begin
		for i:=0 to bufmax do
			buffer[i]:=chr(trunc(256*random));
		bufupd:=100;
	end;

	bufupd:=bufupd-1;


	{ ***** RS232 -> RS485 ***** }
	
	ipqclear(rs485);

	{ transmit buffer out of rs232 port }

	setrts(rs232,true);
	bwrite(rs232,buffer:bufmax+1);
	while opqcount(rs232)<>0 do;
	loadtimer(0,rtsdel); while readtimer(0)<>0 do;	{ extra delay }
	setrts(rs232,false);

	{ receive data from rs485 port and compare with buffer }
	
	loadtimer(1,100);		{ a simple timeout }
	i:=0;
	repeat
		if ipqcount(rs485)<>0 then
		begin
			loadtimer(1,100);
			bread(rs485,ch);
			if ch<>buffer[i] then
				errcnt1:=errcnt1+1;
			i:=i+1;
		end;
	until (i>bufmax) or (readtimer(1)=0);

	if i<>bufmax+1 then
		errcnt2:=errcnt2+1;

	if ipqcount(rs485)<>0 then
		errcnt3:=errcnt3+1;


	{ ***** RS485 -> RS232 ***** }

	ipqclear(rs232);	{ in case KD485 FDX jumper is not in }
	ipqclear(rs485);

	{ transmit buffer out of rs485 port }

	setrts(rs485,true);
	bwrite(rs485,buffer:bufmax+1);
	while opqcount(rs485)<>0 do; 	{ on P3/P4 no extra delay is required }
	setrts(rs485,false);	

	{ receive data from rs232 port and compare with buffer }
	
	loadtimer(1,100);		{ a simple timeout }
	i:=0;
	repeat
		if ipqcount(rs232)<>0 then
		begin
			loadtimer(1,100);
			bread(rs232,ch);
			if ch<>buffer[i] then begin
				errcnt4:=errcnt4+1;
				{writeln(port1,#$0D,#$0A,'i=',i,' b=',ord(buffer[i]):2:h,' c=',ord(ch):2:h);}
			end;
			i:=i+1;
		end;
	until (i>bufmax) or (readtimer(1)=0);

	if i<>bufmax+1 then
		errcnt5:=errcnt5+1;

	if ipqcount(rs232)<>0 then
		errcnt6:=errcnt6+1;

	write(term,'pass=',pass:7:0,'  err=',errcnt1:5,errcnt2:5,errcnt3:5,errcnt4:5,errcnt5:5,errcnt6:5,#$0d);
	pass:=pass+1;

until ipqcount(term)<>0;

end.

