program kdtest1;
{$L-,O-}
{

Test program for KD485 Mode 2.

On P4, simulates a RS485 Master, by regularly polling for data.

On P1, any data retrieved from the RS232 device is displayed.

Requires RTS and DTR to be DISABLED as automatic handshakes on P4.

P4 has a K485-99 in it.

It is assumed that KD485 has address='A', lead-in byte='L'.

It is also assumed that the KD485 M2 timeout is 5ms.  We wait for a bit 
longer than that when waiting for the RS232 device's data, otherwise we might
emit the next poll while the KD485 is returning the data!

The Master polls the KD485 every rs485po ms.

This program can operate in two modes: if checkq=false it keeps polling 
the KD485 for data all the time. If checkq=true it always checks the KD485 
queue size and only retrieves the data if there is more than qmin bytes 
in there.

MOD RECORD

6/7/95	Created.



}


const 
	checkq	= true ;	{ true if we are to check Q size before getting data }
	qmin	= 1;		{ min # of bytes to get, if checkq=true }
	kd485 	= port4;
	term  	= port1;

	rs485po = 100;		{ poll RS485 device every n ms }
	qpoll	= 25;		{ poll for qsize every n ms, if checkq=true }
	rsptim  = 20;		{ timeout for response, must be > kd485 M2 timeout }

var i,qsize:integer;
	ch:char;
	hskcontrol:sethsktype;
	getit,succ:boolean;
	respbuf: array[0..10] of char;	{ big enough for the queue size response }


procedure setrts(p:text; state:boolean);
{ set RTS on port p to specified state }
var	ret:integer;
	hskcontrol:sethsktype;
begin
	with hskcontrol do begin
		hppc_port:=p;
		rts_out:=state; 		{ set RTS to specified state }
		dtr_out:=true; 			{ required for K485 power }
		cdo_out:=false;			{ not used }
	end;
	ret:=sethsk(hskcontrol);	{ execute the new states }
end;



begin

getit:=true;

setrts(kd485,false);

writeln(term,'Running, any key to stop ...');

repeat

	{ RS485 device port: grab what there is in the KD485 RX queue }

	if readtimer(1)=0 then
	begin

		loadtimer(1,rs485po);

		if checkq then
		begin

			{ we loop here until response is what we want }

			getit:=false;	{ don't read KD485 unless response is valid }

			repeat

				{ wait a bit, for KD485's driver to turn off at end of 
				  its last response - reqd at low baud rates only }
				loadtimer(4,4); while readtimer(4)<>0 do;

				loadtimer(3,qpoll);

				{ transmit 0xfe queue space poll }
				setrts(kd485,true);
				write(kd485,'L',#$FE);
				while opqcount(kd485)<>0 do;
				setrts(kd485,false);

				{ discard everything until ':' recd }
				ch:='-';
				loadtimer(2,10);
				repeat
					if ipqcount(kd485)<>0 then
					begin
						bread(kd485,ch);
						loadtimer(2,10);
					end;
				until (ch=':') or (readtimer(2)=0);

				{ get the rest of the response }
				i:=0;
				loadtimer(2,10);
				repeat
					if ipqcount(kd485)<>0 then
					begin
						bread(kd485,respbuf[i]);
						i:=i+1;
						loadtimer(2,10);
					end;
				until ( i>=6 ) or (readtimer(2)=0);

				{ parse the decimal number out of it and check it for size }
				qsize:=0;
				if i>=6 then
				begin
					val( respbuf[0], qsize, i, succ );
					if qsize>=qmin then
						getit:=true;
				end;

				if not getit then
					while readtimer(3)<>0 do;

			until getit or (ipqcount(term)<>0);

		end;

		{ OK, get the data }

		{ wait a bit, like further above }
		loadtimer(4,4); while readtimer(4)<>0 do;

		{ transmit poll for attached RS232 device data }
		setrts(kd485,true);
		write(kd485,'LA');
		while opqcount(kd485)<>0 do;
		setrts(kd485,false);

		{ copy any response to terminal, until timeout }
		loadtimer(2,rsptim);
		repeat
			if (ipqcount(kd485)<>0) and (opqspace(term)<>0) then
			begin
				bread(kd485,ch);
				bwrite(term,ch);
				loadtimer(2,rsptim);
			end;
		until (readtimer(2)=0) or (ipqcount(term)<>0);

	end;

until ipqcount(term)<>0;

end.

