
/* test.c - a test program for KD485-specific C functions */

/*

MOD RECORD

10/12/94	Created. Similar to test.c for PPC.
17/4/95		testbit8 & testbit16 tests changed to output correct bit order.
					Various KD485-specific features added.
4/5/95		Interrupt tests added.
30/5/95		Break receive tests added.
14/6/95		Interrupt test uses temporary variables.
17/6/95		Port test can output data, for character format testing.
22/6/95		Additional float output tests added.
					Sysdata changed for 3rd version digit being a character.
2/8/95		Updated to use wait_485().
16/6/98		Updated with dprintf(), readhex() and RTC tests. 
					Removed tests for ilscanf - doesn't exist on KD485.


Note that the linkfile needs to include dprintf.obj, readhex.obj, rtc.obj
rtc2.obj as well as kd485.lib.

*/


#include "kd485.h"

#define UCH unsigned char
#define UINT unsigned int

#define term port1		// terminal port

#define w_system 0		// for watchdog wdg_mode function
#define w_user   1


/* prototypes */

void 	iostrtests 	( void );
void 	sumtests 		( void );
void	bittests		( void );
void	rotates			( void );
void 	tastests		( void );
void 	rtctests		( void );
void 	queuetests	( void );
void 	timertests	( void );
void 	led_switch_tests	( void );
void 	floatlongs	( void );
void 	dosysdata		( void );
void 	breaks			( void );
void 	interrupts	( void );
void 	portcfg			( void );
void 	sprintfsscanf		( void );
void 	ilsprintftst		( void );
void	eepromtest	( void );
void	wdogtest		( void );
void	dprintftest ( void );
void	readhextest	( void );


float fred,f1,f2,f3;
char ch;
UCH uch;
int i,j,k,l,m,p;
UINT ui;
unsigned long x;
struct tm mytime;
struct sw_t switches;
struct sp_t setport;
char freadcbuf [11];
char user_ee_area [2000] @ USER_EE_START;

char * daytable [ 7 ] = 
{
"Sunday   ",
"Monday   ",
"Tuesday  ",
"Wednesday",
"Thursday ",
"Friday   ",
"Saturday "
};

char * montable [ 12 ] = 
{
"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"
};

char * porttypes [ 2 ] = 
{
"RS232",
"RS422/485"
};

char * baudtab [ 21 ] =
{
"    30",
"  37.5",
"    50",
"    75",
"   100",
"   110",
" 134.5",
"   150",
"   300",
"   600",
"  1200",
"  2000",
"  2400",
"  3600",
"  4800",
"  7200",
"  9600",
" 19200",
" 38400",
" 57600",
"115200"
};

char * paritytab [ 3 ] =
{
"none",
"even",
" odd"
};

						 
char ostr [] = "Hello wot a day\r\n";
char ostr2 [] = "Hello wot a dayHello wot a dayHello wot a day\r\n";
char sumbufstring [] = "sumbufstring";

char checkbuf[5000];



// Debug function which transmits data to a 115200 baud software UART.
// Like fprintf but you don't specify destination port.
// Maximum # of chars output is 100 - the size of the buffer below.

void dprintf(
	char * t,
	...
	)
{
	va_list a;
	int length;
	char buffer[100];
	va_start(a,t);
	length=vsprintf(buffer,t,a);
	va_end(a);
	dfwritec(buffer,length);
}


// Debug function outputs buffer contents, in hex and ascii

void printbuf ( char * buffer, int length )
{
	int j;
	for (j=0;j<length;j++)
		dprintf( "%02x ", (UCH) buffer[j]);
	dprintf( "\r\n");
	for (j=0;j<length;j++)
	{
		if isprint(buffer[j])
			dprintf( " %c ", (UCH) buffer[j]);
		else
			dprintf( "-- ");
	}
}




int main()
{

	drvctrl(port1,1);	// enable P1 driver (in case it is RS422/485)
	drvctrl(port2,1);	// enable P2 driver (in case it is RS422/485)

	for (;;)
	{
		fprintf( term, "\r\npress any key..");
		ipqclear(term);
		fgetc(term);
		fprintf( term, "\x1b[2J");
		fprintf( term, "Select test function:\r\n\r\n");				
	
		fprintf( term, "a) iostrtests \r\n");
		fprintf( term, "b) sumtests\r\n");
		fprintf( term, "c) bittests\r\n");
		fprintf( term, "d) rotates \r\n");
		fprintf( term, "e) spare\r\n");
		fprintf( term, "f) tastests\r\n");
		fprintf( term, "g) rtctests\r\n");
		fprintf( term, "h) queuetests\r\n");
		fprintf( term, "i) timertests\r\n");
		fprintf( term, "j) led_switch_tests\r\n");
		fprintf( term, "k) floatlongs\r\n");
		fprintf( term, "l) sysdata\r\n");
		fprintf( term, "m) breaks\r\n");
		fprintf( term, "n) interrupts\r\n");
		fprintf( term, "o) portcfg\r\n");
		fprintf( term, "p) sprintfsscanf\r\n");
		fprintf( term, "q) ilsprintf\r\n");
		fprintf( term, "r) eepromtest\r\n");
		fprintf( term, "s) watchdogtest\r\n");
		fprintf( term, "t) dprintf\r\n");
		fprintf( term, "u) read_hex_switch (1998 (Mk 2) PROG only)\r\n");
		fprintf( term, "z) exit to Executive\r\n");
	
		fprintf( term, "\r\n");
		ipqclear(term);
		ch=fgetc(term);
		fputc(term,ch);
	
		switch (ch)
		{
			case 'a':	iostrtests (); break;
			case 'b':	sumtests(); break;
			case 'c':	bittests(); break;
			case 'd':	rotates (); break;
			case 'e':	break;
			case 'f':	tastests(); break;
			case 'g':	rtctests(); break;
			case 'h':	queuetests(); break;
			case 'i':	timertests(); break;
			case 'j':	led_switch_tests(); break;
			case 'k':	floatlongs(); break;
			case 'l':	dosysdata(); break;
			case 'm':	breaks(); break;
			case 'n':	interrupts(); break;
			case 'o':	portcfg(); break;
			case 'p':	sprintfsscanf(); break;
			case 'q':	ilsprintftst(); break;
			case 'r':	eepromtest(); break;
			case 's':	wdogtest(); break;
			case 't':	dprintftest(); break;
			case 'u':	readhextest(); break;
			case 'z':	exit();
	
			default :	break;
		}
	}
}


void iostrtests ()
{

	i=fprintf( term, "\r\nHello World\r\n");
	fprintf( term, "fprintf returned %d\r\n",i);

	fprintf( term, "strlen(17)=%d, strchr(6)=%d\r\n",strlen(ostr),strchr(ostr,'w')-ostr);

	fprintf( term, "calling fputc: ");
	i=fputc(term,'f');
	fprintf( term, "\r\nvalue from fputc=%04xh\r\n",i);

	fprintf( term, "calling fgetc, type a char: ");
	ch=fgetc(term);
	fputc(term,ch);
	fprintf( term, "\r\nvalue from fgetc=%02xh\r\n",(int)ch);

	fprintf( term, "testing fwritec, length=%d\r\n",strlen(ostr));
	i=fwritec( term, ostr, strlen(ostr) );
	fprintf( term, "fwritec returns %d\r\n", i);
	fwritec(term,"Hello wot a day\r\n", 17);

	fprintf( term, "testing freadc, type in 10 chars\r\n");
	i=freadc( term, freadcbuf, 10 );
	fprintf( term, "freadc returns %d\r\n", i);
	freadcbuf[10]=0;
	fprintf( term, "freadc string=%s\r\n",freadcbuf);

	fprintf( term, "testing fgetcnd, type a char\r\n");
	while ( ipqcount(term)==0 );
	i=fgetcnd(term);
	fprintf( term, "fgetcnd returns %04x\r\n", i);
	fprintf( term, "there were %d char(s) in the queue\r\n", ipqcount(term));
	ipqclear(term);


}


void sumtests ()
{

	fprintf( term, "\r\n");
	fprintf( term, "testing sumbuf, string length = %d, sumcheck (29) = ", strlen(sumbufstring) );
	uch=sumbuf( sumbufstring, strlen(sumbufstring) );
	fprintf( term, "%02x\r\n", uch);

	fprintf( term, "testing xorbuf, xor sumcheck (0f) = ");
	uch=xorbuf( sumbufstring, strlen(sumbufstring) );
	fprintf( term, "%02x\r\n", uch);

	fprintf( term, "testing crcbuf, crc (0684) = ");
	ui=crcbuf( sumbufstring, strlen(sumbufstring), -1 );
	fprintf( term, "%04x\r\n", ui);

}


void bittests ()
{
	fprintf( term, "\r\n");
	fprintf( term, "testing testbit16 (aa55) : ");
	for (i=15; i>=0; i--)
		fprintf( term, "%1d",testbit16 (i,0xaa55));
	fprintf( term, "\r\ntesting testbit8 (55) : ");
	for (i=7; i>=0; i--)
		fprintf( term, "%1d",testbit8 (i,0x55));
	fprintf( term, "\r\n");

}


void rotates ()
{
	fprintf( term, "\r\n");
	fprintf( term, "testing rotleft16 (0x8003) :\r\n");
	for (j=0; j<16; j++)
	{
		ui=rotleft16(j,0x8003);
		for (i=15; i>=0; i--) fprintf( term, "%1d",testbit16 (i,ui));
		fprintf( term, "\r\n");
	}
	fprintf( term, "testing rotright16 (0x8003) :\r\n");
	for (j=0; j<16; j++)
	{
		ui=rotright16(j,0x8003);
		for (i=15; i>=0; i--) fprintf( term, "%1d",testbit16 (i,ui));
		fprintf( term, "\r\n");
	}
	fprintf( term, "testing rotleft8 (0x83) :\r\n");
	for (j=0; j<8; j++)
	{
		uch=rotleft8(j,0x83);
		for (i=7; i>=0; i--)	fprintf( term, "%1d",testbit8 (i,uch));
		fprintf( term, "\r\n");
	}
	fprintf( term, "testing rotright8 (0x83) :\r\n");
	for (j=0; j<8; j++)
	{
		uch=rotright8(j,0x83);
		for (i=7; i>=0; i--)	fprintf( term, "%1d",testbit8 (i,uch));
		fprintf( term, "\r\n");
	}

}


void tastests ()
{

	static char * tas;

	fprintf( term, "\r\n");
	tas=&ch;
	ch=0;
	fprintf( term,  "test and set function: initial call returns %02x\r\n",testset(tas) );
	fprintf( term,  "test and set function:    next call returns %02x\r\n",testset(tas) );
	fprintf( term,  "test and set function:    next call returns %02x\r\n",testset(tas) );
	fprintf( term, "\r\n");

}





/*

Utility to get a line from P1 into a buffer @s.  Maxlen is the max length.
Echoes everything up to (but not including) the CR, and backspace editing
is supported.  Nulls are discarded at input.
The string in the buffer is null terminated, ready for scanf etc.
Returns # of valid chars typed, including the null terminator.

*/

int getline ( char * s, int maxlen )
{

	char ch;
	int i;

	i=0;

		do
		{
			do
			{
				ch=fgetc(term);	// discard nulls
			}
			while ( ch == 0x00 );

			if ( ch=='\b' )
			{
				if (i>0)
				{
					fprintf( term, "\b \b" );
					i--;
				}
			}
			else
			{
				if ( ch=='\r' )
				{
					s[i++]=0x00;
					break;
				}
				fputc(term,ch);
				s[i++]=ch;
			}
		}
		while ( i < maxlen );

	return(i);

}



void rtctests ()
{

/*

This function tests the DS1302 Real Time Clock. It uses both the lower level
RTC access functions and also the normal setrtc() and getrtc(). The lower
level functions are required to access the 31-byte nonvolatile RAM.

The DS1302 has 8 "clock" registers and 31 general purpose "RAM" registers.
The RAM is not used by the KD485 firmware. It can be accessed one byte at a
time, or in a 31-byte block move which is faster.

To access a clock register, use address of 0..7.
To access a RAM register, use address of 32..62.

The trickle charger (reg=8) is disabled in this test but could be used with
a rechargeable cell. The user program would have to enable the trickle charge
since the KD485 firmware does not do this.

This function can also be used to set the RTC prior to shipment, and to
verify Year2000 compliance.

*/

	UCH tst;
	int addr,errcnt,val;
	UINT seed;
	char tstbuf [31];
	char tstbuf2 [31];

	fprintf( term, "\r\nDisabling DS1302 trickle charge function...");

	rtc_write_byte ( 8, 0 );	// Anything other than 1010xxxx disables charger

	fprintf( term, "\r\nTesting RTC 31-byte RAM (byte access) ");
	errcnt=0;

	for ( addr=32; addr<=62; addr++ )
	{
		fputc( term, '.' );
		for ( j=0; j<256; j++ )
		{
		 	rtc_write_byte ( addr, (UCH) j );
		 	tst=rtc_read_byte ( addr );
			if ( tst != (UCH) j )
			{
				errcnt++;
				fprintf( term, "\r\nError @ addr=%2d, write=%02x, read=%02x\r\n", addr-32, (UCH) j, tst );
			}
		}
	}

	fprintf( term, "\r\nErrors=%d", errcnt );

	fprintf( term, "\r\nTesting RTC 31-byte RAM (block access, random data)\r\n");
	errcnt=0;

	for ( seed=0; seed<20; seed++ )
	{

		srand(seed);

		for ( addr=0; addr<31; addr++ )		// Fill test buffer with random data
			tstbuf[addr]= ( UCH ) rand();

		fputc( term, 'w');
		rtc_write_block_ram( tstbuf );		// Write buffer to RAM

		for ( addr=0; addr<31; addr++ )		// Clear readback buffer before readback
			tstbuf2[addr]=0;

		fputc( term, 'r');
		rtc_read_block_ram( tstbuf2 );		// Read RTC into readback buffer

		for ( addr=0; addr<31; addr++ )		// Compare buffers
			if ( tstbuf[addr] != tstbuf2[addr] ) errcnt++;

	}

	for ( addr=0; addr<31; addr++ )		// Finally zero the RAM
		tstbuf[addr]=0;
	fputc( term, 'z');
	rtc_write_block_ram( tstbuf );

	fprintf( term, "\r\nErrors=%d", errcnt );

	fprintf( term, "\r\nPresent RTC clock state (byte read)  = ");
	for ( i=0; i<=7; i++ )
		fprintf( term, "%02x ", rtc_read_byte(i) );

	fprintf( term, "\r\nPresent RTC clock state (block read) = ");
	rtc_read_block_clock( tstbuf );
	for ( i=0; i<=7; i++ )
		fprintf( term, "%02x ", (UCH) tstbuf[i] );

	fprintf( term, "\r\nSetting RTC to 23:59:50 31/12/1999...\r\n");

	tstbuf[0]=0x50;
	tstbuf[1]=0x59;
	tstbuf[2]=0x23;
	tstbuf[3]=0x31;
	tstbuf[4]=0x12;
	tstbuf[5]=0x07;
	tstbuf[6]=0x00;  // gets loaded below, to avoid compiler warning
	tstbuf[7]=0x00;

	tstbuf[6] = tstbuf[6]+0x99;

	rtc_write_block_clock( tstbuf );

	// Display the 8 RTC clock regs until a key is pressed

	do
	{
	fprintf( term, "\r(press any key to stop) RTC = ");
	rtc_read_block_clock( tstbuf );
	for ( i=0; i<=7; i++ )
		fprintf( term, "%02x ", (UCH) tstbuf[i] );
	loadtimer(1,200); while (readtimer(1));  // wait a bit
	}
	while ( ipqcount( term ) == 0 );
	ipqclear( term );

	fprintf( term, "\r\nRepeating with setrtc() and getrtc() " );

	mytime.tm_year=99;
	mytime.tm_mon=11;
	mytime.tm_mday=30;
	mytime.tm_hour=23;
	mytime.tm_min=59;
	mytime.tm_sec=50;
	mytime.tm_wday=6;

	fprintf( term, "\r\ncalling setrtc, ");
	i=setrtc(&mytime);
	fprintf( term, "return= %d\r\n",i);

	fprintf( term, "calling getrtc (any key to interrupt)..\r\n");

	do
	{
		i=getrtc(&mytime);

		fprintf( term, "time= %04d %s %02d %02d:%02d:%02d %s\r",
						mytime.tm_year+1900,
						montable[mytime.tm_mon],
						mytime.tm_mday+1,
						mytime.tm_hour,
						mytime.tm_min,
						mytime.tm_sec,
						daytable[mytime.tm_wday]);

		loadtimer(1,200); while (readtimer(1));  // wait a bit
	}
	while ( ipqcount( term ) == 0 );
	ipqclear( term );

	// Enable RTC test with user-specified date/time

redo:

	fprintf( term, "\r\n\r\nRTC test with user-specified parameters\r\n");

	fprintf( term, "\r\nYear (1950..2049) = " );
	getline ( tstbuf, 10 );
	sscanf( tstbuf, "%d", &val );
	mytime.tm_year=val-1900;

	fprintf( term, "\r\nMonth (1..12)     = " );
	getline ( tstbuf, 10 );
	sscanf( tstbuf, "%d", &val );
	mytime.tm_mon=val-1;

	fprintf( term, "\r\nDayOfMonth (1..31)= " );
	getline ( tstbuf, 10 );
	sscanf( tstbuf, "%d", &val );
	mytime.tm_mday=val-1;

	fprintf( term, "\r\nDayOfWeek (1..7)  = " );
	getline ( tstbuf, 10 );
	sscanf( tstbuf, "%d", &val );
	mytime.tm_wday=val-1;

	fprintf( term, "\r\nHours (0..23)     = " );
	getline ( tstbuf, 10 );
	sscanf( tstbuf, "%d", &val );
	mytime.tm_hour=val;

	fprintf( term, "\r\nMinutes (0..59)   = " );
	getline ( tstbuf, 10 );
	sscanf( tstbuf, "%d", &val );
	mytime.tm_min=val;

	fprintf( term, "\r\nSeconds (0..59)   = " );
	getline ( tstbuf, 10 );
	sscanf( tstbuf, "%d", &val );
	mytime.tm_sec=val;

	i=setrtc(&mytime);
	fprintf( term, "\r\nsetrtc() return= %d",i);
	if (i) fprintf( term, "(error)" );

	fprintf( term, "\r\ncalling getrtc (any key to interrupt)..\r\n");

	do
	{
		i=getrtc(&mytime);

		fprintf( term, "time= %04d %s %02d %02d:%02d:%02d %s\r",
						mytime.tm_year+1900,
						montable[mytime.tm_mon],
						mytime.tm_mday+1,
						mytime.tm_hour,
						mytime.tm_min,
						mytime.tm_sec,
						daytable[mytime.tm_wday]);

		loadtimer(1,200); while (readtimer(1));  // wait a bit
	}
	while ( ipqcount( term ) == 0 );
	ipqclear( term );

	fprintf( term, "\r\n\r\nRepeat test? (y|Y) " );
	ch=fgetc( term );
	fputc( term, ch );
	if ( toupper(ch) == 'Y' ) goto redo;

}


void queuetests()
{

	fprintf( term, "\r\n");

	fprintf( term, "testing opqcount (p2): ");
	fwritec( port2, ostr2, strlen(ostr2) );
	for (i=0; i<25; i++)
		fprintf( term, "%d ",opqcount(port2));

	fprintf( term, "\r\n");
	fprintf( term, "testing opqspace (p2): ");
	fwritec( port2, ostr2, strlen(ostr2) );
	for (i=0; i<25; i++)
		fprintf( term, "%d ",opqspace(port2));
	
	fprintf( term, "\r\n");
	fprintf( term, "testing ipqcount (p1): press a key 20 times\r\nipqcount(p1)=");
	i=0;
	while ( i < 20 )
	{
		j = ipqcount(term);
		if ( j != i ) 
		{
			fprintf( term, "%d ",j);
			i=j;
		}
	}
	fprintf( term, "\r\nipqclear returns %d ",ipqclear(term));
	fprintf( term, "\r\nipqcount=%d\r\n",ipqcount(term));

}


void timertests()
{

	fprintf( term, "\r\n");

	fprintf( term, "loading all 8 timers with 65535 ms..\r\n");
	for (i=0; i<8; i++)
	{
		j=loadtimer(i,65535);
		if ( j!=0 ) fprintf( term, "error return (%u) on timer %d\r\n",j,i);
	}
	fprintf( term, "reading (any key to interrupt) ..\r\n");

	do
	{
		j=0;	/* j is a flag which (if=0) tells us that ALL timers have dec'd to 0 */
		for (i=0; i<8; i++)
		{
			k=readtimer(i);
			fprintf( term, "t%d=%05u ",i,k);
			if ( k != 0 ) j=1;
		}
		fprintf( term, "\r");
		for (m=0; m<10000; m++);	// slow down output to terminal
	}
	while ( (j!=0) && (ipqcount(term)==0) );

	fprintf( term, "\r\n");

}


void 	led_switch_tests	( )
{

	fprintf( term, "\r\n");
	fprintf( term, "display switch status & copy switch to LED (any key to continue) ..\r\n");

	fprintf( term, "\r\n(dont press switch for > 1 sec)\r\n");

	while ( ipqcount(term) == 0 )
	{
		getsw(&switches);
		fprintf( term, "sw1=%d\r",switches.sw_sw1);
		uled ( switches.sw_sw1 );
		for (m=0; m<1000; m++);
	}

}


void floatlongs ()
{
	fprintf( term, "\r\n");
	for (x=0L; x<200L; x+=50)
		fprintf( term, "hello world, x= %lu\r\n",x);

	for (fred=0; fred<0.05; fred+=0.01)
		fprintf( term, "fred=%f\r\n",fred);

	for (fred=0; fred<0.05; fred+=0.01)
		fprintf( term, "fred=%f, fred/2.15=%f\r\n",fred,fred/2.15);

	for (fred=0; fred<6.282; fred+=0.628)
	{
		f1=sin(fred);
		f2=cos(fred);
		f3=tan(fred);
		fprintf( term, "\r\nx=%f, sin=%f, cos=%f, tan=%f\r\n",fred,f1,f2,f3);
		f1=sinh(fred);
		f2=cosh(fred);
		f3=tanh(fred);
		fprintf( term, "x=%f, sinh=%f, cosh=%f, tanh=%f\r\n",fred,f1,f2,f3);
		f1=sqrt(fred);
		f2=exp(fred);
		f3=pow(fred,2.2*fred);
		fprintf( term, "x=%f, sqrt=%f, exp=%f, pow(x,2.2x)=%f\r\n",fred,f1,f2,f3);
	}

	fprintf( term, "\r\nTesting %%e %%g  %%f with various size numbers:\r\n" );

	fprintf( term, "With %%e 23.45678 outputs as %e\r\n", 23.45678 );
	fprintf( term, "With %%g 23.45678 outputs as %g\r\n", 23.45678 );
	fprintf( term, "With %%f 23.45678 outputs as %f\r\n", 23.45678 );

	fprintf( term, "With %%e 234567.8 outputs as %e\r\n", 234567.8 );
	fprintf( term, "With %%g 234567.8 outputs as %g\r\n", 234567.8 );
	fprintf( term, "With %%f 234567.8 outputs as %f\r\n", 234567.8 );

	fprintf( term, "With %%e 2345678000 outputs as %e\r\n", 2345678000.0 );
	fprintf( term, "With %%g 2345678000 outputs as %g\r\n", 2345678000.0 );
	fprintf( term, "With %%f 2345678000 outputs as %f\r\n", 2345678000.0 );

	fprintf( term, "With %%e 2.345678e+20 outputs as %e\r\n", 2.345678e+20 );
	fprintf( term, "With %%g 2.345678e+20 outputs as %g\r\n", 2.345678e+20 );
	fprintf( term, "With %%f 2.345678e+20 outputs as %f\r\n", 2.345678e+20 );

	fprintf( term, "With %%e 23.45678e-10 outputs as %e\r\n", 23.45678e-10  );
	fprintf( term, "With %%g 23.45678e-10 outputs as %g\r\n", 23.45678e-10  );
	fprintf( term, "With %%30.25f 23.45678e-10 outputs as %30.25f\r\n", 23.45678e-10  );

	fprintf( term, "With %%e -2.345678e-20 outputs as %e\r\n", -2.345678e-20 );
	fprintf( term, "With %%g -2.345678e-20 outputs as %g\r\n", -2.345678e-20 );
	fprintf( term, "With %%30.25f -2.345678e-20 outputs as %30.25f\r\n", -2.345678e-20 );

}


void dosysdata ()
{

	int bit;
	UCH p1type, p2type;
	char sdb [50];	// sysdata buffer needs to be only about 20-30 bytes

	i=sysdata(0);
	fprintf( term, "\r\nsysdata(0) returns %d\r\n",i);
	sysdata( sdb );
	for (ui=0; ui<i; ui++)
		fprintf( term, "%02x ", (UCH) sdb[ui]);

	fprintf(term,"\r\n\r\nKD485 version %d.%d%c\r\n",(UCH)sdb[0],(UCH)sdb[1],sdb[2]);
	fprintf(term,"firmware date %s\r\n",&sdb[3]);
	fprintf(term,"%d ports\r\n",(UCH)sdb[13]);
	fprintf(term,"Serial number %u\r\n",(UCH)sdb[15]+(256*(UCH)sdb[14]));  // H8 byte order!

	fprintf(term,"Installed options byte: ");
	for (bit=7; bit>=0; bit--)
		fprintf( term, "%1d", testbit8(bit,(UCH)sdb[16]));

	p1type=testbit8(0,(UCH)sdb[16]);
	p2type=testbit8(1,(UCH)sdb[16]);

	fprintf( term," (Port 1 = %s, Port 2 = %s)", porttypes[p1type],
																							 porttypes[p2type] );

}


/*

	Function which can be used instead of fgetc, and which can receive breaks
	as well as normal characters.
	If a break is received, it returns a null (0x00) and ccount is written
	with a nonzero value representing the break duration expressed as the number
	of character periods for which the break condition persisted.
	If a normal character is received, it returns the character (0x00..0xff)
	and ccount is written with zero (0x0000).

	Limitation: if there are some nulls already in the RX queue before the break
	condition is received, these will be taken as a part of the break condition,
	as will be any nulls (and also the first non-null character already in the
	RX queue) that are received after the break condition has ended.

	Limitation: because any *valid char will clear the framing error flag, this
	function needs to be called fairly regularly, otherwise a break condition
	might not be detected if it is soon followed by valid data.

	Limitation: when this function removes all the nulls from the RX queue, it
	assumes that is the end of the break condition, and returns.  But if this
	function is entered very shortly after the start of the break condition, it
	will (having removed the few nulls then present in the RX queue) decide	that
	is the end of it, although the actual break condition will in fact persist,
	in which case a subsequent call to this function will return another "break
	detected".  The only way around this is a baud-rate-dependent timeout, or
	something involving direct reading of the UART RX input pin; both of these
	are messy.  The function breaks() below demonstrates this limitation, if
	the break condition lasts a few hundred ms or longer.
	
	Breaks belong to the middle ages, anyway, because all sorts of transient
	line conditions could produce a break condition, so its presence cannot be
	relied on to indicate anything that really matters.

	This function uses the handy property of the H8 UARTs that a break condition
	appears in the UART RX buffer as a null accompanied by a framing error, and
	nulls are received for as long as the break condition persists. Different
	types of UART do different things; e.g. some return just *one null, etc.

*/

char bfgetc( int pt, int * ccount )
{

	char ch;
	UCH * errorflags;
	int count;

	errorflags = geterrs();			// get address of KD485 error flags

	while (ipqcount(pt)==0);		// wait for something to arrive

  ch=fgetcnd(pt);			 				// look at it, without removing it from RX queue

	// If not a null then this char was received before any break condition, so
	// we remove it and return with it, regardless of any current framing error.

	if (ch)
	{
		*ccount=0;
		return(fgetc(pt));
	}

	// We have a null at the head of the RX queue

	if (errorflags[pt] & 0x10)
	{

		// We have a break condition at the head of the RX queue.
		// We remove all contiguous nulls from the queue, while counting them.

		errorflags[pt] &= 0xe0;		// clear framing error flag
		count=0;

		while ( ipqcount(pt) && (fgetcnd(pt)==0) )
		{
			fgetc(pt);							// discard each null
			count++;
		}

		if ( ipqcount(pt) )
			fgetc(pt);			 				// and discard the last few bits of the break!

		*ccount=count;						// load break duration

	}
	else
	{

		*ccount=0;								// A null but no framing error - normal null
		fgetc(pt);

	}

	return(0);									// return with a null

}



void breaks ()
{

	int breaklen;
	char ch;

	fprintf( term, "\r\noutputting 500ms break on P2, brkout returns %d",brkout(port2,500));

	fprintf( term, "\r\nwaiting to receive a break from P2 (10 secs) ");
	loadtimer(0,10000);
	while (readtimer(0))
	{
		if (ipqcount(port2))
		{
			loadtimer(1,200); while (readtimer(1));  // wait a bit
			ch=bfgetc( port2, &breaklen );
			if (breaklen)
				fprintf( term, "\r\ngot a break, %d char periods long", breaklen );
			else
				fprintf( term, "\r\ngot 0x%02x", (UCH)ch );
		}
	}

	fprintf( term, "\r\nwaiting to receive a break from P1 (10 secs) ");
	loadtimer(0,10000);
	while (readtimer(0))
	{
		if (ipqcount(port1))
		{
			loadtimer(1,200); while (readtimer(1));  // wait a bit
			ch=bfgetc( port1, &breaklen );
			if (breaklen)
				fprintf( term, "\r\ngot a break, %d char periods long", breaklen );
			else
				fprintf( term, "\r\ngot 0x%02x", (UCH)ch );
		}
	}

	fprintf( term, "\r\noutputting 500ms break on P1, brkout returns %d",brkout(port1,500));

	ipqclear(port1);		// clean-up, in case of loopback
	ipqclear(port2);		// clean-up, in case of loopback

}


/*

INTERRUPT TESTS

To fully understand these, you should read the Hitachi H8/300 Hardware and
Programming Manuals.

*/


// ISRs for interrupts() below.

volatile static unsigned long long_value;
volatile static float float_value;

static void * oldvec;
static char char_store;

static UCH RDR0 @ 0xffdd;
static UCH SSR0	@ 0xffdc;


interrupt void our_timer_isr ( void )
{

	// Interrupts are disabled here, and we must not re-enable them.  This 
	// means, e.g., no I/O.

	// We do what we want to do - not much in this case.

	long_value++;
	float_value+=0.25;

	// and now we call the ROM-based KD485 system ISR

#asm
	push r0

	; Because the system ISR ends with 'rte', we cannot simply 'call' it; we
	; have to push an extra word on the stack. This will be popped by the 'rte'
	; into the program counter.  Hmmm.  Clive Sinclair would have been proud.

	mov.w #ret_here, r0
	push r0

	; Call the KD485 system timer ISR.

	mov.w @_oldvec, r0
	jsr @r0

	; We return here.  The CCR is undefined now, but that does not matter
	; because at the end of this C function is another 'rte' which restores
	; CCR to its pre-interrupt state.

ret_here:

	pop r0
#endasm

}


interrupt void our_rxchar_isr ( void )
{

	/*

	This is what we have to do

	mov.b @RDR0, r1l	; Get RX char into R1L
	bclr #6, @SSR0		; Clear 'RX char pending' bit

	but we do it in C:

	*/

	char_store = RDR0;
	SSR0 &= 0xbf;

}


void interrupts()
{

	// Temp variables, used to make sure that we don't try to read those 
	// modified by the ISR *while they are being modified.  This could happen
	// with any multi-byte variable

	long temp_long_value;
	float temp_float_value;

	// Initialise variables modified by ISR *before ISR starts running

	long_value=0L;
	float_value=0.0;

	/*

	This first interrupt demo is a little more involved, because not only
	do we get the interrupt to go to *our ISR, but we *then call the old system
	ISR (the one in the ROM) as well.  This ensures that system functions, e.g.
	decrementing of the	timers, etc, still continue.

	If one is hooking-up one of the *other H8/300 timers (none of which are
	used by the KD485 system), one can have an ISR which is like the one
	further	below, where *everything is inside *our ISR.

	We need di() and ei() around setvec() to ensure that a timer interrupt
	cannot occur between the vector being set and the variable oldvec being
	loaded, because oldvec is used by the ISR.

	*/

	// First - hook-up vector # 5 - this is the 1kHz timer int from FRT_OCIA.

	fprintf( term, "\r\nHooking-up our_timer_isr into 1kHz timer chain (vector 5) .."
								 "\r\nAny key to stop\r\n" );

	di();
	oldvec = setvec( 5, our_timer_isr );
	ei();

	// We output the value of the long counter at 1 sec intervals, until a key

	ipqclear(term);

	do
	{
		loadtimer(0,1000);
		di();
		temp_long_value=long_value;
		temp_float_value=float_value;
		ei();
		fprintf( term, "\rlong = %10lu, float = %10.2f", temp_long_value, temp_float_value );
		while (readtimer(0));
	}
	while ( ipqcount(term) == 0 );

	ipqclear(term);

	// Restore original timer interrupt vector

	fprintf( term, "\r\nRestoring original timer interrupt vector ..\r\n");

	setvec( 5, oldvec );

	/*

	Here we hook-up the UART 0 (port 1) receive character ready interrupt, and
	display the chars on the terminal (also on port 1).

	Since we are not calling the old ISR, we need to clear the interrupt at
	its source; with this interrupt that is done by removing the char from the
	UART.

	*/

	fprintf( term, "\r\nHooking-up our_rxchar_isr into vector 15 ..");

	di();
	oldvec = setvec( 15, our_rxchar_isr );
	ei();

	fprintf( term, "\r\nAll keys should echo, until CR\r\n" );

	char_store=0;

	for (;;)
	{
		if ( char_store )
		{
			fputc(term,char_store);
			if ( char_store=='\r' )
				break;
			char_store=0;
		}
	}

	fprintf( term, "\r\nRestoring original UART interrupt vector ..\r\n");

	setvec( 15, oldvec );

	ipqclear(term);

}


void displayport ( int pt )
{
int retcode;
	setport.sp_port=pt;
	retcode=getportcfg(&setport);
	fprintf( term, "read: p%d rt=%d, bd=%s, py=%s, bw=%d, sb=%d, rxx=%d, txx=%d\r\n",
					pt,
					retcode,
					baudtab[setport.sp_brate],
					paritytab[setport.sp_pty],
					setport.sp_bword+5,
					setport.sp_sbits+1,
					setport.sp_rxxhs,
					setport.sp_txxhs );
}


void portcfg()
{

	int i,pt,brate,pty,bword,sbits,test;
	char ch;

	fprintf( term, "\r\nDo you want to output test data from P2 (y/n): ");
	ch=toupper(fgetc(term));
	fputc(term,ch);

	test = ( ch=='Y' ) ? 1 : 0;

	fprintf( term, "\r\ncurrent port config:\r\n");
	for (p=1; p<=TOTAL_PORTS; p++)
		displayport(p);
	fprintf( term, "\r\n");

	/* 
	Try to set p2 to all possible combinations of baud rate & bits/word.
	We dont do p1 because that is what our terminal is connected to ! 
	*/

	for (pt=2; pt<=TOTAL_PORTS; pt++)				// redundant if only 2 ports
	{
		for (brate=0; brate<=20; brate++)			// all baud rates
		{
			for (pty=0; pty<3; pty++)						// all parity values
			{
				for (bword=2; bword<4; bword++)		// all bits/word values
				{
					for (sbits=0; sbits<2; sbits++)	// all bits/word values
					{
						setport.sp_port=pt;
						setport.sp_brate=brate;
						setport.sp_bword=bword;
						setport.sp_pty=pty;
						setport.sp_sbits=sbits;

						// Define all 6 handshake values, even the 'reserved' ones
						setport.sp_rxrtshs=0;
						setport.sp_rxdtrhs=0;
						setport.sp_rxxhs=0;
						setport.sp_txctshs=0;
						setport.sp_txdsrhs=0;
						setport.sp_txxhs=1;						// enable TX xon/xoff

						i=setportcfg(&setport);

						fprintf( term, " set: p%d rt=%d, bd=%s, py=%s, bw=%d, sb=%d\r\n",
														pt,
														i,
														baudtab[setport.sp_brate],
														paritytab[setport.sp_pty],
														setport.sp_bword+5,
														setport.sp_sbits+1
														);

						// and then display the result of getportcfg()
						displayport(pt);

						// If test data output selected, do it, until a key is pressed
						if (test)
						{
							while (ipqcount(term)==0)
							{
								// enable driver
								drvctrl(port2,1);
								// output a string
								fwritec( port2, (char*)ostr, sizeof(ostr) );
								// disable driver (when string all sent)
								while ( opqcount(port2) ); wait_485(port2,0);
								// report '.' to terminal
								fputc(term,'.');
								// dump any P2 RX data, in case loopback plug is fitted!
								ipqclear(port2);
								// wait a bit with driver OFF, so one can see it on a scope
								loadtimer(0,5*(21-brate)); while (readtimer(0));
							}
							fprintf( term, "\r\n" );
							ipqclear(term);
						}

					}
				}
			}
		}
	}

}


void sprintfsscanf()
{

	UINT ui;

	static char sprintfbuf [37];
	static char sscanfbuf [37];

	fprintf( term, "\r\ntesting sprintf ...\r\n");

	for (ui=0; ui<sizeof(sprintfbuf); ui++)
		sprintfbuf[ui]='_';

	i=sprintf( sprintfbuf, "hello %d %ld %f", 25, 250000, 12345.54321 );
	fprintf( term, "\r\nsprintf return=%d",i);

	fprintf( term, "\r\n");
	for (ui=0; ui<sizeof(sprintfbuf); ui++)
		fprintf( term, "%02x", (UCH) sprintfbuf[ui]);
	fprintf( term, "\r\n");
	for (ui=0; ui<sizeof(sprintfbuf); ui++)
		fprintf( term, "%02c", (UCH) sprintfbuf[ui]);
 
	fprintf( term, "\r\n\r\ntesting sscanf ...\r\n");

	for (ui=0; ui<sizeof(sscanfbuf); ui++)
		sscanfbuf[ui]='_';

	sscanf( sprintfbuf, "%s %d %ld %f", sscanfbuf, &i, &x, &fred );
	
	fprintf( term, "\r\n");
	for (ui=0; ui<sizeof(sscanfbuf); ui++)
		fprintf( term, "%02x", (UCH) sscanfbuf[ui]);
	fprintf( term, "\r\n");
	for (ui=0; ui<sizeof(sscanfbuf); ui++)
		fprintf( term, "%02c", (UCH) sscanfbuf[ui]);

	fprintf( term, "\r\n\r\nint=%d, long=%ld, float=%f",i,x,fred );
 	
	fprintf( term, "\r\n\r\n");

}


/*
We test the "il.." version of printf. This is an int- and long-only
version of the full version.  The full version is in the KD485.LIB
library.  The "il" version is in the on-chip ROM, runs faster, and doesn't
add any size to the user program.
*/

void ilsprintftst()
{

	static char sprintfbuf [37];

	ilfprintf( term, "\r\ntesting ilsprintf ...\r\n");

	for (ui=0; ui<sizeof(sprintfbuf); ui++)
		sprintfbuf[ui]='_';

	i=ilsprintf( sprintfbuf, "hello %d %ld %f", 25, 250000, 12345.54321 );
	ilfprintf( term, "\r\nsprintf return=%d",i);

	ilfprintf( term, "\r\n");
	for (ui=0; ui<sizeof(sprintfbuf); ui++)
		ilfprintf( term, "%02x", (UCH) sprintfbuf[ui]);
	ilfprintf( term, "\r\n");
	for (ui=0; ui<sizeof(sprintfbuf); ui++)
		ilfprintf( term, "%02c", (UCH) sprintfbuf[ui]);
 
	ilfprintf( term, "\r\n\r\n");

}




void eepromtest()
{

	UINT ui;

	static char eebuf [2000];
	int errcnt = 0;

	fprintf( term, "\r\n\r\nRandom fill on 2000-byte user eeprom area (block mode).\r\n");
	fprintf( term, "generating random numbers .. \r\n");
	srand(0);
	for (ui=0; ui<2000; ui++)
		eebuf[ui]= ( UCH ) rand();

	fprintf( term, "block write to eeprom .. ");
	i=nvwrblk ( eebuf, 0, 2000 );
	fprintf( term, "done, return code = %d\r\n",i);

	for (ui=0; ui<2000; ui++)
		eebuf[ui]=0;	/* in case block read does nothing ! */

	fprintf( term, "block read from eeprom (memcpy) .. ");
	memcpy(eebuf,user_ee_area,2000);
	fprintf( term, "done\r\n");

	fprintf( term, "comparing .. ");
	srand(0);
	for (ui=0; ui<2000; ui++)
		if ( eebuf[ui] != ( char ) rand() ) errcnt++;
	fprintf( term, "done, errors=%d\r\n",errcnt);

	fprintf( term, "\r\nRandom fill on 2000-byte user eeprom area (byte mode).\r\n");
	fprintf( term, "generating random numbers .. \r\n");
	srand(1);		/* different seed */
	for (ui=0; ui<2000; ui++)
		eebuf[ui]= ( UCH ) rand();

	fprintf( term, "writing to eeprom ");
	for (ui=0; ui<2000; ui++)
	{
		i=nvwrblk ( &eebuf[ui], ui, 1 );
		if (i) fprintf(term, "error, addr=%d, retcode=%d\r\n",ui,i);
		if ( !(ui % 100) ) fputc(term,'.');
	}
	fprintf( term, "done, last write return code = %d\r\n",i);

	fprintf( term, "verifying eeprom .. ");
	for (ui=0; ui<2000; ui++)
	{
		if ( user_ee_area[ui] != eebuf[ui] ) fprintf(term, "error, addr=%d\r\n",ui);
	}
	fprintf( term, "done, last read return code = %d\r\n",i);

	fprintf( term, "\r\nclearing eeprom to zeroes .. ");
	for (ui=0; ui<2000; ui++)
		eebuf[ui]=0;
	i=nvwrblk ( eebuf, 0, 2000 );
	fprintf( term, "done, return code = %d\r\n",i);

}



void wdogtest()
{

	fprintf( term, "\r\nTaking over watchdog from system ..\r\n");

	// 0 = system pulses watchdog, 1 = user program has to do it
	wdg_mode(w_user);

	fprintf( term, "waiting for 3 secs while pulsing watchdog ..\r\n");

	loadtimer(0,3000);			// test for 3s (watchdog trips after about 1.6s)
	while ( readtimer(0) )
	{
 		wdg_pulse();					// pulse the watchdog
		loadtimer(1,200);			// every 200ms
		while ( readtimer(1) );
		fputc(term,'.');
	}

	fprintf( term, " we are still here, so it worked.\r\n");

	fprintf( term, "Returning watchdog pulsing to system ..\r\n");

	wdg_mode(w_system);

	loadtimer(0,3000);			// test for 3s as above, but dont pulse watchdog
	while ( readtimer(0) )
	{
		loadtimer(1,200);
		while ( readtimer(1) );
		fputc(term,'.');
	}

	fprintf( term, " we are still here, so it worked.\r\n");

	// finally, make the watchdog trip

	fprintf( term, "Testing watchdog trip, by taking it over and not pulsing it ..\r\n");
	fprintf( term, "(each dot represents 200ms, there should be about eight)\r\n");

	ipqclear(term);	// get rid of any keystrokes
	wdg_mode(w_user);	// back to 'user' pulsing

	// somewhere in this loop the watchdog will reboot the unit

	do
	{
		loadtimer(1,200);
		while ( readtimer(1) );
		fputc(term,'.');
	}
	while ( ipqcount(term)==0 );	// allow a key to interrupt it

}



void dprintftest()
{

	fprintf( term, "\r\nOutputting data to debug port at 115200 baud ..");
	fprintf( term, "\r\nPress any key to stop ");

	do
	{
		loadtimer(1,200);
		while ( readtimer(1) );
 		dprintf( "ABCDEFGHIJKLMNOPQRSTUVWXYZ-0123456789\r\n" );
		fputc(term,'.');
	}
	while ( ipqcount(term)==0 );	// allow a key to interrupt it

}


void readhextest()
{

	fprintf( term, "\r\nHex Switch test, press any key to stop\r\n");

	do
	{
		loadtimer(1,200);
		while ( readtimer(1) );
		fprintf( term, "value = %d (decimal)\r", readhex() );
	}
	while ( ipqcount(term)==0 );

	fprintf( term, "\r\n");

}


